<?php

// IPplan v4.31
// Aug 24, 2001
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// test if bounds overlap, ie. group can interact with subnet
function TestDuplicateBounds($ds, $boundsaddr, $boundssize, $grp) {

    $result=&$ds->ds->Execute("SELECT boundsaddr
                               FROM bounds
                               WHERE (($boundsaddr BETWEEN boundsaddr AND 
                                  boundsaddr + boundssize - 1) OR
                                  ($boundsaddr < boundsaddr AND 
                                  $boundsaddr+$boundssize > 
                                      boundsaddr + boundssize - 1)) AND
                                  grp=".$ds->ds->qstr($grp));

    if ($result->FetchRow()) {
       return 1;
    }
}

require_once("../ipplanlib.php");
require_once("../adodb/adodb.inc.php");
require_once("../class.dbflib.php");
require_once("../layout/class.layout");
require_once("../auth.php");

$auth = new BasicAuthenticator(ADMINREALM, REALMERROR);

$auth->addUser(ADMINUSER, ADMINPASSWD);

// And now perform the authentication
$auth->authenticate();

// set language
isset($_COOKIE["ipplanNoAuth"]) && myLanguage($_COOKIE['ipplanLanguage']);

//setdefault("window",array("bgcolor"=>"white"));
//setdefault("table",array("cellpadding"=>"0"));
//setdefault("text",array("size"=>"2"));

$title=my_("Add authority boundary to a group");
newhtml($p);
$w=myheading($p, $title, "admin.php");

// explicitly cast variables as security measure against SQL injection
list($size, $grp, $ipaddr) = myRegister("I:size S:grp S:ipaddr");
$formerror="";

$ds=new IPplanDbf() or myError($w,$p, my_("Could not connect to database"));

if ($_POST) {

    if (!$ipaddr)
        $formerror .= my_("Boundary address may not be blank")."\n";
    else if (testIP($ipaddr))
        $formerror .= my_("Invalid boundary address")."\n";
    else if (!$size)
        $formerror .= my_("Size may not be zero")."\n";
    else if (TestDuplicateBounds($ds, inet_aton($ipaddr), $size, $grp))
        $formerror .= my_("Boundary cannot be created - overlaps with existing boundary")."\n";
    else if ($size > 1) {
        if (TestBaseAddr(inet_aton3($ipaddr), $size)) {
            $formerror .= my_("Invalid base address")."\n";
        }
    }

    if (!$formerror) {
        $base=inet_aton($ipaddr);
        $ds->DbfTransactionStart();

        // the fact that the range is unique prevents the range
        // being added to more than one area!
        $result=&$ds->ds->Execute("INSERT INTO bounds
                (boundsaddr, boundssize, grp)
                VALUES
                ($base, $size, ".$ds->ds->qstr($grp).")");

        if ($result) {
            $ds->DbfTransactionEnd();
            insert($w,textbr(my_("Boundary created")));
        }
        else {
            $formerror .= my_("Boundary could not be created")."\n";
        }
    }
}

myError($w,$p, $formerror, FALSE);
 
// display opening text
insert($w,heading(3, "$title."));

insert($w,text(my_("Create a new authority boundary limit by entering the base (network) address. The boundary will limit the members of the group to only be able to create new subnets in this range, thus preventing the user from creating networks outside their area of authority.")));
insert($w,block("<p>"));
insert($w,textbr(my_("A group with no boundary limits (the default) can create networks with no limitations.")));

insert($w, $f2 = form(array("name"=>"ENTRY",
                            "method"=>"post",
                            "action"=>$_SERVER["PHP_SELF"])));

$result=$ds->GetGrps();

$lst=array();
while($row = $result->FetchRow()) {
   $col=$row["grp"];
   $lst["$col"]=$row["grpdescrip"];
}
if (empty($lst)) {
   myError($w,$p, my_("You first need to create some groups!"));
}

insert($f2,textbrbr(my_("Group")));
insert($f2,selectbox($lst,
                  array("name"=>"grp")));

insert($f2,textbrbr(my_("Authority boundary start address")));

insert($f2,input_text(array("name"=>"ipaddr",
                           "value"=>"$ipaddr",
                           "size"=>"15",
                           "maxlength"=>"15")));

insert($f2,textbrbr(my_("Authority size/mask")));

insert($f2,selectbox(array("4"=>"255.255.255.252/30 - 4 hosts",
                           "8"=>"255.255.255.248/29 - 8 hosts",
                           "16"=>"255.255.255.240/28 - 16 hosts",
                           "32"=>"255.255.255.224/27 - 32 hosts",
                           "64"=>"255.255.255.192/26 - 64 hosts",
                           "128"=>"255.255.255.128/25 - 128 hosts",
                           "256"=>"255.255.255.0/24 - 256 hosts (class C)",
                           "512"=>"255.255.254.0/23 - 512 hosts",
                           "1024"=>"255.255.252.0/22 - 1k hosts",
                           "2048"=>"255.255.248.0/21 - 2k hosts",
                           "4096"=>"255.255.240.0/20 - 4k hosts",
                           "8192"=>"255.255.224.0/19 - 8k hosts",
                           "16384"=>"255.255.192.0/18 - 16k hosts",
                           "32768"=>"255.255.128.0/17 - 32k hosts",
                           "65536"=>"255.255.0.0/16 - 64k hosts (class B)",
                           "16777216"=>"255.0.0.0/8 - 16m hosts (class A)"),
                     array("name"=>"size"),
                     $size));

insert($f2,generic("br"));
insert($f2,submit(array("value"=>my_("Submit"))));
insert($f2,freset(array("value"=>my_("Clear"))));

printhtml($p);

?>
